@@ -3,7 +3,7 @@
 /*			     TPM X509 ECC					*/
 /*			     Written by Ken Goldman				*/
 /*		       IBM Thomas J. Watson Research Center			*/
-/*            $Id: X509_ECC.c 1594 2020-03-26 22:15:48Z kgoldman $		*/
+/*            $Id: X509_ECC.c 1658 2021-01-22 23:14:01Z kgoldman $		*/
 /*										*/
 /*  Licenses and Notices							*/
 /*										*/
@@ -55,7 +55,7 @@
 /*    arising in any way out of use or reliance upon this specification or any 	*/
 /*    information herein.							*/
 /*										*/
-/*  (c) Copyright IBM Corp. and others, 2019 - 2020				*/
+/*  (c) Copyright IBM Corp. and others, 2019 - 2021				*/
 /*										*/
 /********************************************************************************/
 
@@ -109,10 +109,11 @@ X509AddSigningAlgorithmECC(
     // If the desired hashAlg definition wasn't found...
     if(hashDef->hashAlg != scheme->details.any.hashAlg)
 	return 0;
-    
+
     switch(scheme->scheme)
 	{
-	  case ALG_ECDSA_VALUE:
+#if ALG_ECDSA
+	  case TPM_ALG_ECDSA:
 	    // Make sure that we have an OID for this hash and ECC
 	    if((hashDef->ECDSA)[0] != ASN1_OBJECT_IDENTIFIER)
 		break;
@@ -123,6 +124,7 @@ X509AddSigningAlgorithmECC(
 	    ASN1StartMarshalContext(ctx);
 	    ASN1PushOID(ctx, hashDef->ECDSA);
 	    return ASN1EndEncapsulation(ctx, ASN1_CONSTRUCTED_SEQUENCE);
+#endif	// ALG_ECDSA
 	  default:
 	    break;
 	}
