@@ -3,7 +3,7 @@
 /*			    Hash structure definitions  			*/
 /*			     Written by Ken Goldman				*/
 /*		       IBM Thomas J. Watson Research Center			*/
-/*            $Id: CryptHash.h 1594 2020-03-26 22:15:48Z kgoldman $		*/
+/*            $Id: CryptHash.h 1658 2021-01-22 23:14:01Z kgoldman $		*/
 /*										*/
 /*  Licenses and Notices							*/
 /*										*/
@@ -55,7 +55,7 @@
 /*    arising in any way out of use or reliance upon this specification or any 	*/
 /*    information herein.							*/
 /*										*/
-/*  (c) Copyright IBM Corp. and others, 2016 - 2020				*/
+/*  (c) Copyright IBM Corp. and others, 2016 - 2021				*/
 /*										*/
 /********************************************************************************/
 
@@ -88,16 +88,7 @@ typedef struct sequenceMethods {
     SMAC_DATA_METHOD          data;
     SMAC_END_METHOD           end;
 } SMAC_METHODS;
-
-#if 0   /* libtpms changed begin (undefined behavior of macro expansion) */
-#define SMAC_IMPLEMENTED (defined CC_MAC || defined CC_MAC_Start)
-#else
-# if defined TPM_CC_MAC || defined TPM_CC_MAC_Start
-#  define SMAC_IMPLEMENTED 1
-# else
-#  define SMAC_IMPLEMENTED 0
-# endif
-#endif  /* libtpms changed end */
+#define SMAC_IMPLEMENTED (CC_MAC || CC_MAC_Start)
 
 /* These definitions are here because the SMAC state is in the union of hash states. */
 
@@ -108,33 +99,76 @@ typedef struct tpmCmacState {
     TPM2B_IV                iv;     // IV buffer
     TPM2B_SYM_KEY           symKey;
 } tpmCmacState_t;
+
 typedef union SMAC_STATES {
 #if ALG_CMAC
     tpmCmacState_t          cmac;
 #endif
     UINT64                  pad;
 } SMAC_STATES;
+
 typedef struct SMAC_STATE {
     SMAC_METHODS            smacMethods;
     SMAC_STATES             state;
 } SMAC_STATE;
-typedef union
-{
+
 #if ALG_SHA1
-    tpmHashStateSHA1_t         Sha1;
+#   define  IF_IMPLEMENTED_SHA1(op)     op(SHA1, Sha1)
+#else
+#   define  IF_IMPLEMENTED_SHA1(op)
 #endif
 #if ALG_SHA256
-    tpmHashStateSHA256_t       Sha256;
+#   define  IF_IMPLEMENTED_SHA256(op)     op(SHA256, Sha256)
+#else
+#   define  IF_IMPLEMENTED_SHA256(op)
 #endif
 #if ALG_SHA384
-    tpmHashStateSHA384_t       Sha384;
+#   define  IF_IMPLEMENTED_SHA384(op)     op(SHA384, Sha384)
+#else
+#   define  IF_IMPLEMENTED_SHA384(op)
 #endif
 #if ALG_SHA512
-    tpmHashStateSHA512_t       Sha512;
+#   define  IF_IMPLEMENTED_SHA512(op)     op(SHA512, Sha512)
+#else
+#   define  IF_IMPLEMENTED_SHA512(op)
 #endif
 #if ALG_SM3_256
-    tpmHashStateSM3_256_t      Sm3_256;
+#   define  IF_IMPLEMENTED_SM3_256(op)     op(SM3_256, Sm3_256)
+#else
+#   define  IF_IMPLEMENTED_SM3_256(op)
 #endif
+#if ALG_SHA3_256
+#   define  IF_IMPLEMENTED_SHA3_256(op)     op(SHA3_256, Sha3_256)
+#else
+#   define  IF_IMPLEMENTED_SHA3_256(op)
+#endif
+#if ALG_SHA3_384
+#   define  IF_IMPLEMENTED_SHA3_384(op)     op(SHA3_384, Sha3_384)
+#else
+#   define  IF_IMPLEMENTED_SHA3_384(op)
+#endif
+#if ALG_SHA3_512
+#   define  IF_IMPLEMENTED_SHA3_512(op)     op(SHA3_512, Sha3_512)
+#else
+#   define  IF_IMPLEMENTED_SHA3_512(op)
+#endif
+
+/* SHA512 added kgold */
+#define FOR_EACH_HASH(op)		    \
+    IF_IMPLEMENTED_SHA1(op)		    \
+    IF_IMPLEMENTED_SHA256(op)		    \
+    IF_IMPLEMENTED_SHA384(op)		    \
+    IF_IMPLEMENTED_SHA512(op)		    \
+    IF_IMPLEMENTED_SM3_256(op)		    \
+    IF_IMPLEMENTED_SHA3_256(op)		    \
+    IF_IMPLEMENTED_SHA3_384(op)		    \
+    IF_IMPLEMENTED_SHA3_512(op)
+
+#define HASH_TYPE(HASH, Hash)   tpmHashState##HASH##_t  Hash;
+
+typedef union
+{
+    FOR_EACH_HASH(HASH_TYPE)
     // Additions for symmetric block cipher MAC
 #if SMAC_IMPLEMENTED
     SMAC_STATE                 smac;
@@ -146,6 +180,7 @@ typedef union
     uint32_t             align;
 #endif
 } ANY_HASH_STATE;
+
 typedef ANY_HASH_STATE *PANY_HASH_STATE;
 typedef const ANY_HASH_STATE    *PCANY_HASH_STATE;
 #define ALIGNED_SIZE(x, b) ((((x) + (b) - 1) / (b)) * (b))
@@ -198,21 +233,10 @@ typedef struct _HASH_METHODS
     HASH_STATE_IMPORT_METHOD    *copyIn;    // Copy a hash block to a proper hash
     // context
 } HASH_METHODS, *PHASH_METHODS;
-#if ALG_SHA1
-TPM2B_TYPE(SHA1_DIGEST, SHA1_DIGEST_SIZE);
-#endif
-#if ALG_SHA256
-TPM2B_TYPE(SHA256_DIGEST, SHA256_DIGEST_SIZE);
-#endif
-#if ALG_SHA384
-TPM2B_TYPE(SHA384_DIGEST, SHA384_DIGEST_SIZE);
-#endif
-#if ALG_SHA512
-TPM2B_TYPE(SHA512_DIGEST, SHA512_DIGEST_SIZE);
-#endif
-#if ALG_SM3_256
-TPM2B_TYPE(SM3_256_DIGEST, SM3_256_DIGEST_SIZE);
-#endif
+
+#define HASH_TPM2B(HASH, Hash)  TPM2B_TYPE(HASH##_DIGEST, HASH##_DIGEST_SIZE);
+
+FOR_EACH_HASH(HASH_TPM2B)
 
 /* When the TPM implements RSA, the hash-dependent OID pointers are part of the HASH_DEF. These
    macros conditionally add the OID reference to the HASH_DEF and the HASH_DEF_TEMPLATE. */
