@@ -3,7 +3,7 @@
 /*		Implementation of cryptographic functions for hashing.		*/
 /*			     Written by Ken Goldman				*/
 /*		       IBM Thomas J. Watson Research Center			*/
-/*            $Id: CryptHash.c 1594 2020-03-26 22:15:48Z kgoldman $		*/
+/*            $Id: CryptHash.c 1658 2021-01-22 23:14:01Z kgoldman $		*/
 /*										*/
 /*  Licenses and Notices							*/
 /*										*/
@@ -55,7 +55,7 @@
 /*    arising in any way out of use or reliance upon this specification or any 	*/
 /*    information herein.							*/
 /*										*/
-/*  (c) Copyright IBM Corp. and others, 2016 - 2020				*/
+/*  (c) Copyright IBM Corp. and others, 2016 - 2021				*/
 /*										*/
 /********************************************************************************/
 
@@ -68,41 +68,24 @@
 #include "CryptHash_fp.h"
 #include "CryptHash.h"
 #include "OIDs.h"
-#define HASH_TABLE_SIZE     (HASH_COUNT + 1)
-#if     ALG_SHA1
-HASH_DEF_TEMPLATE(SHA1, Sha1);
-#endif
-#if     ALG_SHA256
-HASH_DEF_TEMPLATE(SHA256, Sha256);
-#endif
-#if     ALG_SHA384
-HASH_DEF_TEMPLATE(SHA384, Sha384);
-#endif
-#if     ALG_SHA512
-HASH_DEF_TEMPLATE(SHA512, Sha512);
-#endif
-#if ALG_SM3_256
-HASH_DEF_TEMPLATE(SM3_256, Sm3_256);
-#endif
-HASH_DEF NULL_Def = {{0}};
+
+/* Instance each of the hash descriptors based on the implemented algorithms */
+
+FOR_EACH_HASH(HASH_DEF_TEMPLATE)
+
+/* Instance a null def. */
+
+    HASH_DEF NULL_Def = {{0}};
+
+/* Create a table of pointers to the defined hash definitions */
+
+#define HASH_DEF_ENTRY(HASH, Hash)     &Hash##_Def,
 PHASH_DEF       HashDefArray[] = {
-#if ALG_SHA1
-				  &Sha1_Def,
-#endif
-#if ALG_SHA256
-				  &Sha256_Def,
-#endif
-#if ALG_SHA384
-				  &Sha384_Def,
-#endif
-#if ALG_SHA512
-				  &Sha512_Def,
-#endif
-#if ALG_SM3_256
-				  &Sm3_256_Def,
-#endif
+				  // for each implemented HASH, expands to: &HASH_Def,
+				  FOR_EACH_HASH(HASH_DEF_ENTRY)
 				  &NULL_Def
 };
+
 /* 10.2.13.3	Obligatory Initialization Functions */
 /* 10.2.13.3.1	CryptHashInit() */
 /* This function is called by _TPM_Init() do perform the initialization operations for the
@@ -132,20 +115,20 @@ CryptHashStartup(
 /* 10.2.13.4.2	CryptGetHashDef() */
 /* This function accesses the hash descriptor associated with a hash a algorithm. The function
    returns a pointer to a null descriptor if hashAlg is TPM_ALG_NULL or not a defined algorithm. */
+
 PHASH_DEF
 CryptGetHashDef(
 		TPM_ALG_ID       hashAlg
 		)
 {
-    size_t           i;
-#define HASHES  (sizeof(HashDefArray) / sizeof(PHASH_DEF))
-    for(i = 0; i < HASHES; i++)
+#define GET_DEF(HASH, Hash) case ALG_##HASH##_VALUE: return &Hash##_Def;
+    switch(hashAlg)
 	{
-	    PHASH_DEF p = HashDefArray[i];
-	    if(p->hashAlg == hashAlg)
-		return p;
+	    FOR_EACH_HASH(GET_DEF)
+	  default:
+	    return &NULL_Def;
 	}
-    return &NULL_Def;
+#undef GET_DEF
 }
 /* 10.2.13.4.3	CryptHashIsValidAlg() */
 /* This function tests to see if an algorithm ID is a valid hash algorithm. If flag is true, then
